/**
 * @file DE_NosyncDemi.cpp
 * @author ...
 * ...
 * @@tag DifferenceEquationMultiple (generic)@@namespace:DEtimeStep;DE_NosyncDemi;par:;sync:x|;nosync:@@end tag@@
 */

#include <vle/extension/DifferenceEquation.hpp>

//@@begin:includes@@
#include <iomanip>      // std::setprecision
#include <iostream>
#include <vle/devs/DynamicsDbg.hpp>
//@@end:includes@@

namespace vd = vle::devs;
namespace ve = vle::extension;
namespace vv = vle::value;

namespace DEtimeStep {

class DE_NosyncDemi: public ve::DifferenceEquation::Multiple {
public:
    DE_NosyncDemi(const vd::DynamicsInit& atom, const vd::InitEventList& evts) :
            ve::DifferenceEquation::Multiple(atom, evts)
    {
        y = createVar("y");
        x = createNosync("x");
    }

    virtual ~DE_NosyncDemi()
    {
    }

//@@begin:compute@@
    virtual void compute(const vd::Time& time)
    {
        std::cout << "\t-------------DE_NosyncDemi compute-------------" << std::endl;
        std::cout << std::setprecision(20) << "\ttime:\t" << time << std::endl;
        std::cout << std::setprecision(20) << "\tx(t-0.5):\t" << x(-0.5) << std::endl;
        std::cout << std::setprecision(20) << "\ty(t-1):\t" << y(-1) << std::endl;

        y = y(-1) + x(-0.5);

        std::cout << std::setprecision(20) << "\t-> y(t)=x(t-0.5)+y(t-1):\t" << y() << std::endl;
    }
//@@end:compute@@

//@@begin:initValue@@
    virtual void initValue(const vd::Time& /*time*/)
    {
        y = 0.0;
        std::cout << "\t-------------DE_NosyncDemi init-------------" << std::endl;
        std::cout << std::setprecision(20) << "\ttime-step:\t" << mTimeStep << std::endl;
    }
//@@end:initValue@@

private:
//@@begin:user@@
//@@end:user@@

    Var y;
    Nosync x;
};

} // namespace DEtimeStep

DECLARE_DYNAMICS_DBG(DEtimeStep::DE_NosyncDemi)

