/**
 * @file DE_NosyncDemiHack.cpp
 * @author ...
 * ...
 * @@tag DifferenceEquationMultiple (generic)@@namespace:DEtimeStep;DE_NosyncDemiHack;par:;sync:x|;nosync:@@end tag@@
 */

#include <vle/extension/DifferenceEquation.hpp>

//@@begin:includes@@
#include <iomanip>      // std::setprecision
#include <iostream>
#include <vle/devs/DynamicsDbg.hpp>
//@@end:includes@@

namespace vd = vle::devs;
namespace ve = vle::extension;
namespace vv = vle::value;

namespace DEtimeStep {

class DE_NosyncDemiHack: public ve::DifferenceEquation::Multiple {
public:
    DE_NosyncDemiHack(const vd::DynamicsInit& atom, const vd::InitEventList& evts) :
            ve::DifferenceEquation::Multiple(atom, evts)
    {
        y = createVar("y");
        x = createNosync("x");
    }

    virtual ~DE_NosyncDemiHack()
    {
    }

//@@begin:compute@@
    virtual void compute(const vd::Time& time)
    {
        std::cout << "\t-------------DE_NosyncDemiHack compute-------------" << std::endl;
        std::cout << std::setprecision(20) << "\ttime:\t" << time << std::endl;
        std::cout << std::setprecision(20) << "\tx(t-1):\t" << x(-1) << std::endl;
        std::cout << std::setprecision(20) << "\ty(t-2):\t" << y(-2) << std::endl;

        y = y(-2) + x(-1);

        std::cout << std::setprecision(20) << "\t-> y(t)=x(t-1)+y(t-2):\t" << y() << std::endl;
    }
//@@end:compute@@

//@@begin:initValue@@
    virtual void initValue(const vd::Time& /*time*/)
    {
        y = 0.0;
        std::cout << "\t-------------DE_NosyncDemiHack init-------------" << std::endl;
        std::cout << std::setprecision(20) << "\ttime-step:\t" << mTimeStep << std::endl;
    }
//@@end:initValue@@

private:
//@@begin:user@@
//@@end:user@@

    Var y;
    Nosync x;
};

} // namespace DEtimeStep

DECLARE_DYNAMICS_DBG(DEtimeStep::DE_NosyncDemiHack)

