/**
  * @file DE_Sync.cpp
  * @author ...
  * ...
  * @@tag DifferenceEquationMultiple (generic)@@namespace:DEtimeStep;class:DE_Sync;par:;sync:x|;nosync:@@end tag@@
  */

#include <vle/extension/DifferenceEquation.hpp>

//@@begin:includes@@
#include <iomanip>      // std::setprecision
#include <iostream>
#include <vle/devs/DynamicsDbg.hpp>
//@@end:includes@@

namespace vd = vle::devs;
namespace ve = vle::extension;
namespace vv = vle::value;

namespace DEtimeStep {

class DE_Sync : public ve::DifferenceEquation::Multiple
{
public:
    DE_Sync(
       const vd::DynamicsInit& atom,
       const vd::InitEventList& evts)
        : ve::DifferenceEquation::Multiple(atom, evts)
    {
        y = createVar("y");
        x = createSync("x");
    }

    virtual ~DE_Sync()
    {}

//@@begin:compute@@
    virtual void compute(const vd::Time& time)
    {
        std::cout << "\t-------------DE_Sync compute-------------" << std::endl;
        std::cout << std::setprecision(20) << "\ttime:\t" << time << std::endl;
        std::cout << std::setprecision(20) << "\tx(t):\t" << x() << std::endl;
        std::cout << std::setprecision(20) << "\ty(t-1):\t" << y(-1) << std::endl;

        y = y(-1) + x();

        std::cout << std::setprecision(20) << "\t-> y(t)=x(t)+y(t-1):\t" << y() << std::endl;
    }
//@@end:compute@@

//@@begin:initValue@@
    virtual void initValue(const vd::Time& /*time*/)
    {
        std::cout << "\t-------------DE_Sync init-------------" << std::endl;
        std::cout << std::setprecision(20) << "\ttime-step:\t" << mTimeStep << std::endl;
        y = 0.0;
    }
//@@end:initValue@@

private:
//@@begin:user@@
//@@end:user@@

    Var y;
    Sync x;
};

} // namespace DEtimeStep

DECLARE_DYNAMICS(DEtimeStep::DE_Sync)

