/**
 * @file DE_counter.cpp
 * @author ...
 * ...
 * @@tag DifferenceEquationMultiple (generic)@@namespace:DEtimeStep;class:DE_counter;par:;sync:;nosync:|@@end tag@@
 */

#include <vle/extension/DifferenceEquation.hpp>

//@@begin:includes@@
#include <iomanip>      // std::setprecision
#include <iostream>
#include <vle/devs/DynamicsDbg.hpp>
//@@end:includes@@

namespace vd = vle::devs;
namespace ve = vle::extension;
namespace vv = vle::value;

namespace DEtimeStep {

class DE_counter: public ve::DifferenceEquation::Multiple {
public:
    DE_counter(const vd::DynamicsInit& atom, const vd::InitEventList& evts) :
            ve::DifferenceEquation::Multiple(atom, evts)
    {
        x = createVar("x");
    }

    virtual ~DE_counter()
    { }

//@@begin:compute@@
    virtual void compute(const vd::Time& time)
    {
        std::cout << "\t-------------DE_counter compute-------------" << std::endl;
        std::cout << std::setprecision(20) << "\t\t\t" << "\ttime:\t" << time << std::endl;
        std::cout << std::setprecision(20) << "\t\t\t" << "\tx(t-1):\t" << x(-1) << std::endl;

        x = x(-1) + 1;

        std::cout << std::setprecision(20) << "\t\t\t" << "\tx(t)=x(t-1)+1:\t" << x() << std::endl;
    }
//@@end:compute@@

//@@begin:initValue@@
    virtual void initValue(const vd::Time& /*time*/)
    {
        std::cout << "\t-------------DE_counter init-------------" << std::endl;
        std::cout << std::setprecision(20) << "\t\t\t" << "\ttime-step:\t" << mTimeStep << std::endl;
        x = 0.0;
    }
//@@end:initValue@@

private:
//@@begin:user@@
//@@end:user@@

    Var x;
};

} // namespace DEtimeStep

DECLARE_DYNAMICS_DBG(DEtimeStep::DE_counter)

